@@ -16,6 +16,7 @@
 package org.gradle.api.plugins.antlr.internal;
 
 import groovy.lang.Closure;
+import org.gradle.api.Action;
 import org.gradle.api.file.SourceDirectorySet;
 import org.gradle.api.internal.file.SourceDirectorySetFactory;
 import org.gradle.api.plugins.antlr.AntlrSourceVirtualDirectory;
@@ -34,12 +35,20 @@ public class AntlrSourceVirtualDirectoryImpl implements AntlrSourceVirtualDirect
         antlr.getFilter().include("**/*.g4");
     }
 
+    @Override
     public SourceDirectorySet getAntlr() {
         return antlr;
     }
 
+    @Override
     public AntlrSourceVirtualDirectory antlr(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getAntlr());
         return this;
     }
+
+    @Override
+    public AntlrSourceVirtualDirectory antlr(Action<? super SourceDirectorySet> configureAction) {
+        configureAction.execute(getAntlr());
+        return this;
+    }
 }
