@@ -25,14 +25,15 @@ import org.gradle.api.logging.LogLevel;
 import org.gradle.api.logging.configuration.ConsoleOutput;
 import org.gradle.api.logging.configuration.LoggingConfiguration;
 import org.gradle.api.logging.configuration.ShowStacktrace;
+import org.gradle.api.logging.configuration.WarningMode;
+import org.gradle.concurrent.ParallelismConfiguration;
 import org.gradle.initialization.BuildLayoutParameters;
 import org.gradle.initialization.CompositeInitScriptFinder;
-import org.gradle.internal.concurrent.DefaultParallelismConfiguration;
 import org.gradle.initialization.DistributionInitScriptFinder;
-import org.gradle.concurrent.ParallelismConfiguration;
 import org.gradle.initialization.UserHomeInitScriptFinder;
 import org.gradle.internal.DefaultTaskExecutionRequest;
 import org.gradle.internal.FileUtils;
+import org.gradle.internal.concurrent.DefaultParallelismConfiguration;
 import org.gradle.internal.installation.CurrentGradleInstallation;
 import org.gradle.internal.installation.GradleInstallation;
 import org.gradle.internal.logging.DefaultLoggingConfiguration;
@@ -94,6 +95,7 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
     private List<File> includedBuilds = new ArrayList<File>();
     private boolean buildScan;
     private boolean noBuildScan;
+    private boolean interactive;
 
     /**
      * {@inheritDoc}
@@ -143,6 +145,22 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
         loggingConfiguration.setConsoleOutput(consoleOutput);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public WarningMode getWarningMode() {
+        return loggingConfiguration.getWarningMode();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setWarningMode(WarningMode warningMode) {
+        loggingConfiguration.setWarningMode(warningMode);
+    }
+
     /**
      * Sets the project's cache location. Set to null to use the default location.
      */
@@ -189,6 +207,7 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
 
     protected StartParameter prepareNewInstance(StartParameter p) {
         prepareNewBuild(p);
+        p.setWarningMode(getWarningMode());
         p.buildFile = buildFile;
         p.projectDir = projectDir;
         p.settingsFile = settingsFile;
@@ -223,6 +242,7 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
         p.setLogLevel(getLogLevel());
         p.setConsoleOutput(getConsoleOutput());
         p.setShowStacktrace(getShowStacktrace());
+        p.setWarningMode(getWarningMode());
         p.profile = profile;
         p.continueOnFailure = continueOnFailure;
         p.offline = offline;
@@ -234,6 +254,7 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
         p.configureOnDemand = configureOnDemand;
         p.setMaxWorkerCount(getMaxWorkerCount());
         p.systemPropertiesArgs = new HashMap<String, String>(systemPropertiesArgs);
+        p.interactive = interactive;
         return p;
     }
 
@@ -748,6 +769,7 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
             + ", configureOnDemand=" + configureOnDemand
             + ", maxWorkerCount=" + getMaxWorkerCount()
             + ", buildCacheEnabled=" + buildCacheEnabled
+            + ", interactive=" + interactive
             + '}';
     }
 
@@ -828,4 +850,23 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
         this.noBuildScan = noBuildScan;
     }
 
+    /**
+     * Returns true when console is interactive.
+     *
+     * @since 4.3
+     */
+    @Incubating
+    public boolean isInteractive() {
+        return interactive;
+    }
+
+    /**
+     * Specifies whether console is interactive.
+     *
+     * @since 4.3
+     */
+    @Incubating
+    public void setInteractive(boolean interactive) {
+        this.interactive = interactive;
+    }
 }
