@@ -19,6 +19,7 @@ package org.gradle.api.artifacts.result;
 import org.gradle.api.Incubating;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.internal.scan.UsedByScanPlugin;
 
 import javax.annotation.Nullable;
 import java.util.Set;
@@ -26,6 +27,7 @@ import java.util.Set;
 /**
  * Represents a component instance in the resolved dependency graph. Provides some basic identity and dependency information about the component.
  */
+@UsedByScanPlugin
 @Incubating
 // Should really be named ComponentGraphResult
 public interface ResolvedComponentResult {
