@@ -15,14 +15,14 @@
  */
 package org.gradle.initialization;
 
-import com.google.common.collect.Maps;
 import org.gradle.StartParameter;
 import org.gradle.api.Action;
+import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.UnknownProjectException;
 import org.gradle.api.initialization.ConfigurableIncludedBuild;
-import org.gradle.api.initialization.IncludedBuild;
 import org.gradle.api.initialization.ProjectDescriptor;
 import org.gradle.api.initialization.Settings;
+import org.gradle.api.initialization.dsl.ScriptHandler;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.SettingsInternal;
 import org.gradle.api.internal.file.FileResolver;
@@ -32,22 +32,21 @@ import org.gradle.api.internal.plugins.DefaultObjectConfigurationAction;
 import org.gradle.api.internal.plugins.PluginManagerInternal;
 import org.gradle.api.internal.project.AbstractPluginAware;
 import org.gradle.api.internal.project.ProjectRegistry;
-import org.gradle.vcs.SourceControl;
 import org.gradle.caching.configuration.BuildCacheConfiguration;
-import org.gradle.composite.internal.IncludedBuildFactory;
 import org.gradle.configuration.ScriptPluginFactory;
 import org.gradle.groovy.scripts.ScriptSource;
 import org.gradle.internal.Actions;
-import org.gradle.internal.Cast;
 import org.gradle.internal.resource.TextResourceLoader;
 import org.gradle.internal.scripts.ScriptFileResolver;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
 import org.gradle.plugin.management.PluginManagementSpec;
+import org.gradle.vcs.SourceControl;
 
 import javax.inject.Inject;
 import java.io.File;
-import java.util.Map;
+import java.util.ArrayList;
+import java.util.List;
 
 import static org.gradle.util.NameValidator.asValidName;
 
@@ -67,14 +66,17 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
 
     private final ClassLoaderScope settingsClassLoaderScope;
     private final ClassLoaderScope buildRootClassLoaderScope;
+    private final ScriptHandler scriptHandler;
     private final ServiceRegistry services;
-    private final Map<File, ConfigurableIncludedBuild> includedBuilds = Maps.newLinkedHashMap();
+
+    private final List<IncludedBuildSpec> includedBuildSpecs = new ArrayList<IncludedBuildSpec>();
 
     public DefaultSettings(ServiceRegistryFactory serviceRegistryFactory, GradleInternal gradle,
-                           ClassLoaderScope settingsClassLoaderScope, ClassLoaderScope buildRootClassLoaderScope, File settingsDir,
-                           ScriptSource settingsScript, StartParameter startParameter) {
+                           ClassLoaderScope settingsClassLoaderScope, ClassLoaderScope buildRootClassLoaderScope, ScriptHandler settingsScriptHandler,
+                           File settingsDir, ScriptSource settingsScript, StartParameter startParameter) {
         this.gradle = gradle;
         this.buildRootClassLoaderScope = buildRootClassLoaderScope;
+        this.scriptHandler = settingsScriptHandler;
         this.settingsDir = settingsDir;
         this.settingsScript = settingsScript;
         this.startParameter = startParameter;
@@ -92,10 +94,20 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         return gradle;
     }
 
+    @Override
+    public List<IncludedBuildSpec> getIncludedBuilds() {
+        return includedBuildSpecs;
+    }
+
     public Settings getSettings() {
         return this;
     }
 
+    @Override
+    public ScriptHandler getBuildscript() {
+        return scriptHandler;
+    }
+
     public DefaultProjectDescriptor createProjectDescriptor(DefaultProjectDescriptor parent, String name, File dir) {
         return new DefaultProjectDescriptor(parent, name, dir, getProjectDescriptorRegistry(), getFileResolver(), getScriptFileResolver());
     }
@@ -124,7 +136,7 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         return projectDescriptor;
     }
 
-    public void include(String[] projectPaths) {
+    public void include(String... projectPaths) {
         for (String projectPath : projectPaths) {
             String subPath = "";
             String[] pathElements = removeTrailingColon(projectPath).split(":");
@@ -141,7 +153,7 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         }
     }
 
-    public void includeFlat(String[] projectNames) {
+    public void includeFlat(String... projectNames) {
         for (String projectName : projectNames) {
             createProjectDescriptor(rootProjectDescriptor, projectName,
                 new File(rootProjectDescriptor.getProjectDir().getParentFile(), projectName));
@@ -250,11 +262,6 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         throw new UnsupportedOperationException();
     }
 
-    @Inject
-    protected IncludedBuildFactory getIncludedBuildFactory() {
-        throw new UnsupportedOperationException();
-    }
-
     @Inject
     public PluginManagerInternal getPluginManager() {
         throw new UnsupportedOperationException();
@@ -267,18 +274,12 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
 
     @Override
     public void includeBuild(Object rootProject, Action<ConfigurableIncludedBuild> configuration) {
-        File projectDir = getFileResolver().resolve(rootProject);
-        ConfigurableIncludedBuild build = includedBuilds.get(projectDir);
-        if (build == null) {
-            build = getIncludedBuildFactory().createBuild(projectDir);
-            includedBuilds.put(projectDir, build);
+        if (gradle.getParent() == null) {
+            File projectDir = getFileResolver().resolve(rootProject);
+            includedBuildSpecs.add(new IncludedBuildSpec(projectDir, configuration));
+        } else {
+            throw new InvalidUserDataException(String.format("Included build '%s' cannot have included builds.", getRootProject().getName()));
         }
-        configuration.execute(build);
-    }
-
-    @Override
-    public Map<File, IncludedBuild> getIncludedBuilds() {
-        return Cast.uncheckedCast(includedBuilds);
     }
 
     @Override
