@@ -32,7 +32,6 @@ import org.slf4j.LoggerFactory;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
 
@@ -44,8 +43,8 @@ class NodeState implements DependencyGraphNode {
 
     private final Long resultId;
     private final ComponentState component;
-    private final Set<EdgeState> incomingEdges = new LinkedHashSet<EdgeState>();
-    private final Set<EdgeState> outgoingEdges = new LinkedHashSet<EdgeState>();
+    private final List<EdgeState> incomingEdges = Lists.newLinkedList();
+    private final List<EdgeState> outgoingEdges = Lists.newLinkedList();
     private final ResolvedConfigurationIdentifier id;
 
     private final ConfigurationMetadata metaData;
@@ -90,12 +89,12 @@ class NodeState implements DependencyGraphNode {
     }
 
     @Override
-    public Set<EdgeState> getIncomingEdges() {
+    public List<EdgeState> getIncomingEdges() {
         return incomingEdges;
     }
 
     @Override
-    public Set<EdgeState> getOutgoingEdges() {
+    public List<EdgeState> getOutgoingEdges() {
         return outgoingEdges;
     }
 
@@ -126,7 +125,7 @@ class NodeState implements DependencyGraphNode {
         return metaData.isTransitive();
     }
 
-    public void visitOutgoingDependencies(Collection<EdgeState> target) {
+    public void visitOutgoingDependencies(Collection<EdgeState> target, PendingDependenciesHandler pendingDependenciesHandler) {
         // If this configuration's version is in conflict, don't do anything
         // If not traversed before, add all selected outgoing edges
         // If traversed before, and the selected modules have changed, remove previous outgoing edges and add outgoing edges again with
@@ -165,15 +164,30 @@ class NodeState implements DependencyGraphNode {
             removeOutgoingEdges();
         }
 
-        for (DependencyMetadata dependency : metaData.getDependencies()) {
-            if (isExcluded(resolutionFilter, dependency)) {
-                continue;
+        visitDependencies(resolutionFilter, pendingDependenciesHandler, target);
+
+    }
+
+    protected void visitDependencies(ModuleExclusion resolutionFilter, PendingDependenciesHandler pendingDependenciesHandler, Collection<EdgeState> resultingOutgoingEdges) {
+        boolean isOptionalConfiguration = "optional".equals(metaData.getName());
+        PendingDependenciesHandler.Visitor pendingDepsVisitor =  pendingDependenciesHandler.start(isOptionalConfiguration);
+        try {
+            for (DependencyMetadata dependency : metaData.getDependencies()) {
+                DependencyState dependencyState = new DependencyState(dependency, resolveState.getComponentSelectorConverter());
+                if (isExcluded(resolutionFilter, dependencyState)) {
+                    continue;
+                }
+                if (!pendingDepsVisitor.maybeAddAsPendingDependency(this, dependencyState)) {
+                    EdgeState dependencyEdge = new EdgeState(this, dependencyState, resolutionFilter, resolveState);
+                    outgoingEdges.add(dependencyEdge);
+                    resultingOutgoingEdges.add(dependencyEdge);
+                }
             }
-            EdgeState dependencyEdge = new EdgeState(this, dependency, resolutionFilter, resolveState);
-            outgoingEdges.add(dependencyEdge);
-            target.add(dependencyEdge);
+            previousTraversalExclusions = resolutionFilter;
+        } finally {
+            // we must do this after `previousTraversalExclusions` has been written, or state won't be reset properly
+            pendingDepsVisitor.complete();
         }
-        previousTraversalExclusions = resolutionFilter;
     }
 
     private List<EdgeState> findTransitiveIncomingEdges(boolean hasIncomingEdges) {
@@ -205,7 +219,8 @@ class NodeState implements DependencyGraphNode {
         }
     }
 
-    private boolean isExcluded(ModuleExclusion selector, DependencyMetadata dependency) {
+    private boolean isExcluded(ModuleExclusion selector, DependencyState dependencyState) {
+        DependencyMetadata dependency = dependencyState.getDependencyMetadata();
         if (!resolveState.getEdgeFilter().isSatisfiedBy(dependency)) {
             LOGGER.debug("{} is filtered.", dependency);
             return true;
@@ -213,7 +228,7 @@ class NodeState implements DependencyGraphNode {
         if (selector == ModuleExclusions.excludeNone()) {
             return false;
         }
-        ModuleIdentifier targetModuleId = resolveState.getModuleIdentifierFactory().module(dependency.getRequested().getGroup(), dependency.getRequested().getName());
+        ModuleIdentifier targetModuleId = dependencyState.getModuleIdentifier();
         if (selector.excludeModule(targetModuleId)) {
             LOGGER.debug("{} is excluded from {}.", targetModuleId, this);
             return true;
@@ -236,20 +251,18 @@ class NodeState implements DependencyGraphNode {
         return !incomingEdges.isEmpty();
     }
 
-    private ModuleExclusion getModuleResolutionFilter(List<EdgeState> transitiveEdges) {
-        ModuleExclusion resolutionFilter;
+    private ModuleExclusion getModuleResolutionFilter(List<EdgeState> incomingEdges) {
         ModuleExclusions moduleExclusions = resolveState.getModuleExclusions();
-        if (transitiveEdges.isEmpty()) {
-            resolutionFilter = ModuleExclusions.excludeNone();
-        } else {
-            resolutionFilter = transitiveEdges.get(0).getExclusions(moduleExclusions);
-            for (int i = 1; i < transitiveEdges.size(); i++) {
-                EdgeState dependencyEdge = transitiveEdges.get(i);
-                resolutionFilter = moduleExclusions.union(resolutionFilter, dependencyEdge.getExclusions(moduleExclusions));
-            }
+        ModuleExclusion nodeExclusions = moduleExclusions.excludeAny(metaData.getExcludes());
+        if (incomingEdges.isEmpty()) {
+            return nodeExclusions;
         }
-        resolutionFilter = moduleExclusions.intersect(resolutionFilter, metaData.getExclusions(moduleExclusions));
-        return resolutionFilter;
+        ModuleExclusion edgeExclusions = incomingEdges.get(0).getExclusions();
+        for (int i = 1; i < incomingEdges.size(); i++) {
+            EdgeState dependencyEdge = incomingEdges.get(i);
+            edgeExclusions = moduleExclusions.union(edgeExclusions, dependencyEdge.getExclusions());
+        }
+        return moduleExclusions.intersect(edgeExclusions, nodeExclusions);
     }
 
     public void removeOutgoingEdges() {
@@ -287,4 +300,10 @@ class NodeState implements DependencyGraphNode {
     public void deselect() {
         removeOutgoingEdges();
     }
+
+    void resetSelectionState() {
+        previousTraversalExclusions = null;
+        outgoingEdges.clear();
+        resolveState.onMoreSelected(this);
+    }
 }
