@@ -16,8 +16,8 @@
 
 package org.gradle.api.internal.artifacts.transform;
 
-import com.google.common.hash.HashCode;
 import net.jcip.annotations.ThreadSafe;
+import org.gradle.internal.hash.HashCode;
 import org.gradle.internal.util.BiFunction;
 
 import java.io.File;
@@ -33,4 +33,6 @@ public interface TransformedFileCache {
      * @param transformer The transformer to apply to produce the output, if not already available
      */
     List<File> getResult(File inputFile, HashCode inputsHash, BiFunction<List<File>, File, File> transformer);
+
+    boolean contains(File absoluteFile, HashCode inputsHash);
 }
