@@ -27,17 +27,19 @@ import org.gradle.initialization.BuildIdentity;
 public class DefaultProjectComponentIdentifier implements ProjectComponentIdentifier {
     private final BuildIdentifier buildIdentifier;
     private final String projectPath;
-    private final String displayName;
+    private String displayName;
 
     public DefaultProjectComponentIdentifier(BuildIdentifier buildIdentifier, String projectPath) {
         assert buildIdentifier != null : "build cannot be null";
         assert projectPath != null : "project path cannot be null";
         this.buildIdentifier = buildIdentifier;
         this.projectPath = projectPath;
-        displayName = "project " + fullPath(buildIdentifier, projectPath);
     }
 
     public String getDisplayName() {
+        if (displayName == null) {
+            displayName = "project " + fullPath(buildIdentifier, projectPath);
+        }
         return displayName;
     }
 
@@ -50,6 +52,14 @@ public class DefaultProjectComponentIdentifier implements ProjectComponentIdenti
         return projectPath;
     }
 
+    public String getProjectName() {
+        if (projectPath.equals(":")) {
+            return buildIdentifier.getName();
+        }
+        int index = projectPath.lastIndexOf(':');
+        return projectPath.substring(index + 1);
+    }
+
     @Override
     public boolean equals(Object o) {
         if (this == o) {
@@ -71,14 +81,17 @@ public class DefaultProjectComponentIdentifier implements ProjectComponentIdenti
 
     @Override
     public String toString() {
-        return displayName;
+        return getDisplayName();
     }
 
     private static String fullPath(BuildIdentifier build, String projectPath) {
-        if (build.isCurrentBuild()) {
+        if (build.getName().equals(":")) {
             return projectPath;
+        } else if (projectPath.equals(":")) {
+            return ":" + build.getName();
+        } else {
+            return ":" + build.getName() + projectPath;
         }
-        return ":" + build.getName() + projectPath;
     }
 
     public static ProjectComponentIdentifier newProjectId(IncludedBuild build, String projectPath) {
@@ -86,6 +99,10 @@ public class DefaultProjectComponentIdentifier implements ProjectComponentIdenti
         return new DefaultProjectComponentIdentifier(buildIdentifier, projectPath);
     }
 
+    public static ProjectComponentIdentifier newProjectId(BuildIdentifier buildIdentifier, String projectPath) {
+        return new DefaultProjectComponentIdentifier(buildIdentifier, projectPath);
+    }
+
     public static ProjectComponentIdentifier newProjectId(Project project) {
         BuildIdentifier buildId = ((ProjectInternal) project).getServices().get(BuildIdentity.class).getCurrentBuild();
         return new DefaultProjectComponentIdentifier(buildId, project.getPath());
