@@ -54,7 +54,7 @@ public class DefaultGradleDistribution implements GradleDistribution {
     }
 
     public GradleExecuter executer(TestDirectoryProvider testDirectoryProvider, IntegrationTestBuildContext buildContext) {
-        return new NoDaemonGradleExecuter(this, testDirectoryProvider, version, buildContext);
+        return new NoDaemonGradleExecuter(this, testDirectoryProvider, version, buildContext).withWarningMode(null);
     }
 
     public boolean worksWith(Jvm jvm) {
@@ -145,7 +145,13 @@ public class DefaultGradleDistribution implements GradleDistribution {
     }
 
     public VersionNumber getArtifactCacheLayoutVersion() {
-        if (isSameOrNewer("4.3-rc-1")) {
+        if (isSameOrNewer("4.5.1-rc-1")) {
+            return VersionNumber.parse("2.51");
+        } else if (isSameOrNewer("4.5-rc-1")) {
+            return VersionNumber.parse("2.48");
+        } else if (isSameOrNewer("4.4-rc-1")) {
+            return VersionNumber.parse("2.36");
+        } else if (isSameOrNewer("4.3-rc-1")) {
             return VersionNumber.parse("2.31");
         } else if (isSameOrNewer("4.2-rc-1")) {
             return VersionNumber.parse("2.24");
