@@ -16,37 +16,38 @@
 
 package org.gradle.language.swift.internal;
 
+import org.apache.commons.lang.StringUtils;
 import org.gradle.api.artifacts.ConfigurationContainer;
 import org.gradle.api.internal.file.FileOperations;
 import org.gradle.api.model.ObjectFactory;
+import org.gradle.api.provider.Property;
 import org.gradle.language.swift.SwiftApplication;
 import org.gradle.language.swift.SwiftExecutable;
+import org.gradle.language.swift.SwiftPlatform;
+import org.gradle.nativeplatform.toolchain.internal.NativeToolChainInternal;
+import org.gradle.nativeplatform.toolchain.internal.PlatformToolProvider;
 
 import javax.inject.Inject;
 
 public class DefaultSwiftApplication extends DefaultSwiftComponent implements SwiftApplication {
-    private final DefaultSwiftExecutable debug;
-    private final DefaultSwiftExecutable release;
+    private final ObjectFactory objectFactory;
+    private final Property<SwiftExecutable> developmentBinary;
 
     @Inject
     public DefaultSwiftApplication(String name, ObjectFactory objectFactory, FileOperations fileOperations, ConfigurationContainer configurations) {
         super(name, fileOperations, objectFactory, configurations);
-        debug = objectFactory.newInstance(DefaultSwiftExecutable.class, name + "Debug", objectFactory, getModule(), true, getSwiftSource(), configurations, getImplementationDependencies());
-        release = objectFactory.newInstance(DefaultSwiftExecutable.class, name + "Release", objectFactory, getModule(), false, getSwiftSource(), configurations, getImplementationDependencies());
+        this.objectFactory = objectFactory;
+        this.developmentBinary = objectFactory.property(SwiftExecutable.class);
+    }
+
+    public SwiftExecutable addExecutable(String nameSuffix, boolean debuggable, boolean optimized, boolean testable, SwiftPlatform targetPlatform, NativeToolChainInternal toolChain, PlatformToolProvider platformToolProvider) {
+        SwiftExecutable result = objectFactory.newInstance(DefaultSwiftExecutable.class, getName() + StringUtils.capitalize(nameSuffix), getModule(), debuggable, optimized, testable, getSwiftSource(), getImplementationDependencies(), targetPlatform, toolChain, platformToolProvider);
+        getBinaries().add(result);
+        return result;
     }
 
     @Override
-    public SwiftExecutable getDevelopmentBinary() {
-        return debug;
-    }
-
-    @Override
-    public SwiftExecutable getDebugExecutable() {
-        return debug;
-    }
-
-    @Override
-    public SwiftExecutable getReleaseExecutable() {
-        return release;
+    public Property<SwiftExecutable> getDevelopmentBinary() {
+        return developmentBinary;
     }
 }
