@@ -30,6 +30,7 @@ public class ProgressStartEvent extends CategorisedOutputEvent {
     private final String shortDescription;
     private final String loggingHeader;
     private final String status;
+    private final int totalProgress;
     private final Object buildOperationId;
     private final Object parentBuildOperationId;
     private BuildOperationCategory buildOperationCategory;
@@ -42,6 +43,7 @@ public class ProgressStartEvent extends CategorisedOutputEvent {
                               @Nullable String shortDescription,
                               @Nullable String loggingHeader,
                               String status,
+                              int totalProgress,
                               @Nullable Object buildOperationId,
                               @Nullable Object parentBuildOperationId,
                               BuildOperationCategory buildOperationCategory) {
@@ -52,6 +54,7 @@ public class ProgressStartEvent extends CategorisedOutputEvent {
         this.shortDescription = shortDescription;
         this.loggingHeader = loggingHeader;
         this.status = status;
+        this.totalProgress = totalProgress;
         this.buildOperationId = buildOperationId;
         this.parentBuildOperationId = parentBuildOperationId;
         this.buildOperationCategory = buildOperationCategory;
@@ -80,6 +83,10 @@ public class ProgressStartEvent extends CategorisedOutputEvent {
         return status;
     }
 
+    public int getTotalProgress() {
+        return totalProgress;
+    }
+
     @Override
     public String toString() {
         return "ProgressStart " + description;
