@@ -18,6 +18,7 @@ package org.gradle.api.internal.tasks.testing.worker;
 
 import org.gradle.api.Action;
 import org.gradle.api.internal.classpath.ModuleRegistry;
+import org.gradle.api.internal.tasks.testing.JULRedirector;
 import org.gradle.api.internal.tasks.testing.TestClassProcessor;
 import org.gradle.api.internal.tasks.testing.TestClassRunInfo;
 import org.gradle.api.internal.tasks.testing.TestResultProcessor;
@@ -66,6 +67,7 @@ public class ForkingTestClassProcessor implements TestClassProcessor {
     public void processTestClass(TestClassRunInfo testClass) {
         if (remoteProcessor == null) {
             completion = currentWorkerLease.startChild();
+            JULRedirector.checkDeprecatedProperty(options);
             remoteProcessor = forkProcess();
         }
 
