@@ -20,8 +20,15 @@ import org.gradle.initialization.BuildEventConsumer;
 import org.gradle.internal.progress.BuildOperationDescriptor;
 import org.gradle.internal.progress.BuildOperationListener;
 import org.gradle.internal.progress.OperationFinishEvent;
+import org.gradle.internal.progress.OperationProgressEvent;
 import org.gradle.internal.progress.OperationStartEvent;
-import org.gradle.tooling.internal.provider.events.*;
+import org.gradle.tooling.internal.provider.events.AbstractOperationResult;
+import org.gradle.tooling.internal.provider.events.DefaultFailure;
+import org.gradle.tooling.internal.provider.events.DefaultFailureResult;
+import org.gradle.tooling.internal.provider.events.DefaultOperationDescriptor;
+import org.gradle.tooling.internal.provider.events.DefaultOperationFinishedProgressEvent;
+import org.gradle.tooling.internal.provider.events.DefaultOperationStartedProgressEvent;
+import org.gradle.tooling.internal.provider.events.DefaultSuccessResult;
 
 import java.util.Collections;
 
@@ -43,6 +50,10 @@ class ClientForwardingBuildOperationListener implements BuildOperationListener {
         eventConsumer.dispatch(new DefaultOperationStartedProgressEvent(startEvent.getStartTime(), toBuildOperationDescriptor(buildOperation)));
     }
 
+    @Override
+    public void progress(BuildOperationDescriptor buildOperation, OperationProgressEvent progressEvent) {
+    }
+
     @Override
     public void finished(BuildOperationDescriptor buildOperation, OperationFinishEvent result) {
         eventConsumer.dispatch(new DefaultOperationFinishedProgressEvent(result.getEndTime(), toBuildOperationDescriptor(buildOperation), adaptResult(result)));
